#include <OneWire.h>
#include <DallasTemperature.h>
#include <LiquidCrystal_I2C.h>
#include <SoftwareSerial.h>

#define TEMP_THRESHOLD 29
#define SMOKE_THRESHOLD 500

float temp = 0.0; 
int buzzer = 11;
int oneWireBus = 12;  
int smokeA0 = A0;
OneWire oneWire(oneWireBus);
DallasTemperature sensors(&oneWire);    
LiquidCrystal_I2C lcd(0x27,20,4);
SoftwareSerial SIM900A(2,3);
int flag = 0;
int hasSentMessage = 0;
void SendMessage() {
  lcd.setCursor(2,0);
  lcd.print("Send Message");
  Serial.println ("Sending Message please wait....");
  SIM900A.println("AT+CMGF=1"); 
  delay(1000);
  Serial.println ("Set SMS Number");
  SIM900A.println("AT+CMGS=\"+40743666069\"\r");
  delay(1000);
  Serial.println ("Set SMS Content");
  SIM900A.println("Fire alarm activated!!! Sent STOP to acknowledge");
  delay(100);
  Serial.println ("Done");
  SIM900A.println((char)26);
  lcd.setCursor(2,0);
  lcd.print("Message Sent");
  Serial.println ("Message sent succesfully");
}

void setup(void) {
    SIM900A.begin(9600);
    lcd.init();
    lcd.backlight();
    lcd.setCursor(2,0);
    lcd.print("Alarm System");
    pinMode(smokeA0, INPUT);
    pinMode(buzzer, OUTPUT);
    Serial.begin(9600);                 
    sensors.begin();                   
}

void loop(void) {
    if(flag == 0) {
      int MQ2Sensor = analogRead(smokeA0); 
      sensors.requestTemperatures();
      temp = sensors.getTempCByIndex(0);
      Serial.print("MQ2 SMOKE detected: ");
      Serial.println(MQ2Sensor);
      Serial.print("Temperature is: "); 
      Serial.println(temp);
      if (temp > TEMP_THRESHOLD || MQ2Sensor > SMOKE_THRESHOLD) {
        flag = 1;
        SendMessage();
      }
    delay(1000);
    }
    if(flag == 1) {
      tone(buzzer, 2000);
      String buff = "";
      while(SIM900A.available()) {
        char c = SIM900A.read();
        buff.concat(c);
        delay(20);
      }
      if(buff.length() > 51) {
        buff.remove(0, 50);
        if(buff.startsWith("STOP")== true) {
          flag = 0;
          noTone(buzzer);
          lcd.setCursor(2,0);
          lcd.print("Alarm System");
        }
      }
    }
}
